#include "MediaAVDecoder.h"

MediaAVDecoder::MediaAVDecoder()
{
}
MediaAVDecoder::~MediaAVDecoder()
{
    if (m_avcodec_context)
    {
        avcodec_free_context(&m_avcodec_context);
        m_avcodec_context = nullptr;
    }
}
bool MediaAVDecoder::init(AVFormatContext *m_avformat_ctx, enum AVMediaType type)
{
    int stream_index = av_find_best_stream(m_avformat_ctx, type, -1, -1, NULL, 0);
    if (stream_index < 0)
    {
        return false;
    }

    m_avstream = m_avformat_ctx->streams[stream_index];
    if (!m_avstream)
    {
        printf("stream is empty\n");
        return false;
    }

    m_avcodec = (AVCodec *)avcodec_find_decoder(m_avstream->codecpar->codec_id);
    if (!m_avcodec)
    {
        fprintf(stderr, "Failed to find %s codec\n", av_get_media_type_string(type));
        return false;
    }
    else
    {
        m_avcodec_context = avcodec_alloc_context3(m_avcodec);
        if ((avcodec_parameters_to_context(m_avcodec_context, m_avstream->codecpar)) < 0)
        {
            printf("Failed to copy %s codec parameters to decoder context\n");
            return false;
        }
        // AVFrame生命周期不由编码器关闭，由用户自己管理。如果不设置这个选项，一些frame数据将会丢失导致画面或者声音异常
        AVDictionary *opts = NULL;
        av_dict_set(&opts, "refcounted_frames", "1", 0);
        av_opt_set(m_avcodec_context->priv_data, "tune", "zerolatency", 0);
        if (avcodec_open2(m_avcodec_context, m_avcodec, &opts) < 0)
        {
            printf("Failed to open %s codec\n");
            return false;
        }
    }
    printf("open decoder ok\n");
    return true;
}

int MediaAVDecoder::send_avpacket(MediaAVPacket* pkt)
{
    int ret = 0;
    if (pkt == nullptr)
    {
        ret = avcodec_send_packet(m_avcodec_context, nullptr);
    }
    else
    {
        ret = avcodec_send_packet(m_avcodec_context, pkt->m_pakcet);
    }
    return ret;
}
int MediaAVDecoder::receive_avframe(MediaAVFrame* frame)
{
    auto ret = avcodec_receive_frame(m_avcodec_context, frame->m_frame);
    return ret;
}