#include "SDL2RenderWidget.h"

SDL2RenderWidget::SDL2RenderWidget(QWidget *parent)
	: m_stb_media(new Stb_Media)
{
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER) < 0)
	{
		printf("SDL could not initialize! SDL_Error: %s\n", SDL_GetError());
	}
	SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "1");
 
	m_sdl_window = SDL_CreateWindowFrom((void *)(parent->winId()));
	m_sdl_renderer = SDL_CreateRenderer(m_sdl_window, -1, SDL_RENDERER_ACCELERATED);

	SDL_SetRenderDrawBlendMode(m_sdl_renderer, SDL_BLENDMODE_BLEND);
	if (!m_sdl_window)
	{
		printf("SDL: could not create window - exiting:%s\n", SDL_GetError());
	}
	SDL_CreateThread(SDL2RenderWidget::refresh_default_image, NULL, this);
}

SDL2RenderWidget::~SDL2RenderWidget()
{
	// 销毁 window
	SDL_DestroyWindow(m_sdl_window);
	// 退出 SDL subsystems
	SDL_Quit();
}
void SDL2RenderWidget::set_Size(QSize _size)
{
	m_width = _size.width();
	m_height = _size.height();
	SDL_SetWindowSize(m_sdl_window, _size.width(), _size.height());
}
void SDL2RenderWidget::renderer_default_image()
{
	SDL_WaitEvent(&event);
	if (event.type == REFRESH_EVENT)
	{
		if (!m_sdl_surface)
		{
			m_sdl_surface = m_stb_media->default_logo_surface(logoImageStr);
			m_sdl_texture = SDL_CreateTextureFromSurface(m_sdl_renderer, m_sdl_surface);
			m_stb_media->setFontArgs(m_sdl_renderer, 48, 14);
		}
		int iW, iH, iWidth, iHeight;
		SDL_Rect m_image_rect, m_font_rect;
		SDL_QueryTexture(m_sdl_texture, NULL, NULL, &iW, &iH);
		SDL_GetWindowSize(m_sdl_window, &iWidth, &iHeight);

		m_image_rect.x = iWidth / 2 - iW / 2;
		m_image_rect.y = iHeight / 2 - iH / 2;
		m_image_rect.w = iW;
		m_image_rect.h = iH;

		SDL_RenderClear(m_sdl_renderer);
		SDL_RenderCopy(m_sdl_renderer, m_sdl_texture, NULL, &m_image_rect);

		m_font_rect.x = (iWidth / 2 - iW / 2) - m_stb_media->fc.faceSize;
		m_font_rect.y = iHeight - 50;

		renderer_text_info(m_font_rect, logoStr, default_logo_str);
		SDL_RenderPresent(m_sdl_renderer);
	}
}
void SDL2RenderWidget::update_image(MediaAVFrame *v_frame, v_sws_spec *v_spec, use_opt *_opt)
{
	m_renderer_image_type = media_video_image;

	int nTextureWidth, nTextureHeight;
	// 首先查询当前纹理对象的宽高，如果不符合，那么需要重建纹理对象
	SDL_QueryTexture(m_sdl_texture, nullptr, nullptr, &nTextureWidth, &nTextureHeight);

	if (nTextureWidth != v_frame->m_frame->width || nTextureHeight != v_frame->m_frame->height)
	{
		if (!m_sdl_texture)
		{
			SDL_DestroyTexture(m_sdl_texture);
		}
		m_sdl_texture = SDL_CreateTexture(m_sdl_renderer, SDL_PIXELFORMAT_IYUV, SDL_TEXTUREACCESS_STREAMING,
										  v_frame->m_frame->width, v_frame->m_frame->height);
		SDL_SetWindowSize(m_sdl_window, v_frame->m_frame->width, v_frame->m_frame->height);
		//改变播放器窗口同帧一样大
		emit signals_change_size(v_frame->m_frame->width, v_frame->m_frame->height);
	}
	SDL_GetWindowSize(m_sdl_window, &nTextureWidth, &nTextureHeight);
	SDL_Rect m_src_rect, m_dst_rect, m_font_rect;
	m_src_rect.x = 0;
	m_src_rect.y = 0;
	m_src_rect.w = v_frame->m_frame->width;
	m_src_rect.h = v_frame->m_frame->height;

	switch (_opt->aspect_type)
	{
	case keep_aspect_ratio: // 始终保持纵横比且图像居中
		if (m_src_rect.w < _opt->width | m_src_rect.h < _opt->height)
		{
			m_dst_rect.x = (_opt->width - v_frame->m_frame->width)/2;
			m_dst_rect.y = (_opt->height - v_frame->m_frame->height )/ 2;
			m_dst_rect.w = v_frame->m_frame->width;
			m_dst_rect.h = v_frame->m_frame->height;
		}
		else//否则按照帧大小播放
		{
			m_dst_rect.x = 0;
			m_dst_rect.y = 0;
			m_dst_rect.w = v_frame->m_frame->width;
			m_dst_rect.h = v_frame->m_frame->height;
		}
		break;
	case stretch_aspect_ratio://拉伸模式(画面会糊得妈都不认识)
		m_dst_rect.x = 0;
		m_dst_rect.y = 0;
		m_dst_rect.w = v_frame->m_frame->width;
		m_dst_rect.h = v_frame->m_frame->height;
		break;
	}

	SDL_UpdateYUVTexture(m_sdl_texture,
						 &m_src_rect,
						 v_frame->m_frame->data[0],
						 v_frame->m_frame->linesize[0],
						 v_frame->m_frame->data[1],
						 v_frame->m_frame->linesize[1],
						 v_frame->m_frame->data[2],
						 v_frame->m_frame->linesize[2]);
	SDL_RenderClear(m_sdl_renderer);
	SDL_RenderCopy(m_sdl_renderer, m_sdl_texture, &m_src_rect, &m_dst_rect);
	if (v_spec->info_state)
	{
		m_font_rect.x = 0;
		m_font_rect.y = 0;
		std::string info = "\n" + v_spec->filename + "\n" +
						   v_spec->width + "\n" +
						   v_spec->height + "\n" +
						   v_spec->channels + "\n" +
						   v_spec->sample_rate + "\n" +
						   v_spec->bit_rate + "\n" +
						   v_spec->current_fps_rate + "\n" +
						   v_spec->duration + "\n" +
						   v_spec->pix_fmt + "\n" +
						   v_spec->pkg_fmt_name + "\n";
		renderer_text_info(m_font_rect, info, media_info_str);
	}
	SDL_RenderPresent(m_sdl_renderer);
}

SDL_AudioDeviceID SDL2RenderWidget::openAudioDevice(SDL_AudioSpec *spec)
{
	SDL_AudioSpec have;
	SDL_AudioDeviceID dev = SDL_OpenAudioDevice(NULL, 0, spec, &have, SDL_AUDIO_ALLOW_FORMAT_CHANGE);
	if (dev == 0)
	{
		SDL_Log("Failed to open audio: %s", SDL_GetError());
	}
	else
	{
		if (have.format != spec->format)
		{ /* we let this one thing change. */
			SDL_Log("We didn't get Float32 audio format.");
		}
	}
	SDL_PauseAudioDevice(dev, 0);
	return dev;
}
void SDL2RenderWidget::renderer_text_info(SDL_Rect m_rect, std::string str, font_renderer_type f_type)
{

	switch (f_type)
	{
	case media_info_str:
		if (m_stb_media->fc.faceSize > 20)
		{
			m_stb_media->fc.clearGlyphs();
			m_stb_media->setFontArgs(m_sdl_renderer, 20, 14);
		}
		m_stb_media->fc.drawText(m_rect.x, m_rect.y, str);
		break;
	case media_attachment:

		break;
	case default_logo_str:
		m_stb_media->fc.drawText(m_rect.x, m_rect.y, str);
		break;
	}
}
int SDL2RenderWidget::refresh_default_image(void *opaque)
{
	SDL2RenderWidget *n_sdl = (SDL2RenderWidget *)opaque;
	SDL_Event _event;
	while (true)
	{
		if (n_sdl->m_renderer_image_type)
		{
			_event.type = REFRESH_EVENT;
			SDL_PushEvent(&_event);
			QMetaObject::invokeMethod(
				QApplication::instance(), [=]()
				{ n_sdl->renderer_default_image(); },
				Qt::QueuedConnection);
		}
		SDL_Delay(n_sdl->sd_time);
	}
	_event.type = BREAK_EVENT;
	SDL_PushEvent(&_event);
	return 0;
}
