#include "MediaPlayer.h"

MediaPlayer::MediaPlayer() : m_avformat_ctx(nullptr),
                             m_video_avdecoder(nullptr),
                             m_audio_avdecoder(nullptr),
                             m_render_receive_obj(nullptr),
                             m_sws_spec(nullptr),
                             m_sws_ctx(nullptr),
                             m_use_opt(nullptr),
                             m_video_af_frame(nullptr),
                             m_video_ot_frame(nullptr),
                             m_audio_af_frame(nullptr),
                             m_audio_ot_frame(nullptr)
{
    m_sws_spec = new v_sws_spec;
    m_use_opt = new use_opt;
    m_sws_spec->info_state = true;
}
MediaPlayer::~MediaPlayer()
{
}
char *CharAVPixelFormat[]{
    "AV_PIX_FMT_NONE",
    "AV_PIX_FMT_YUV420P",   ///< planar YUV 4:2:0, 12bpp, (1 Cr & Cb sample per 2x2 Y samples)
    "AV_PIX_FMT_YUYV422",   ///< packed YUV 4:2:2, 16bpp, Y0 Cb Y1 Cr
    "AV_PIX_FMT_RGB24",     ///< packed RGB 8:8:8, 24bpp, RGBRGB...
    "AV_PIX_FMT_BGR24",     ///< packed RGB 8:8:8, 24bpp, BGRBGR...
    "AV_PIX_FMT_YUV422P",   ///< planar YUV 4:2:2, 16bpp, (1 Cr & Cb sample per 2x1 Y samples)
    "AV_PIX_FMT_YUV444P",   ///< planar YUV 4:4:4, 24bpp, (1 Cr & Cb sample per 1x1 Y samples)
    "AV_PIX_FMT_YUV410P",   ///< planar YUV 4:1:0,  9bpp, (1 Cr & Cb sample per 4x4 Y samples)
    "AV_PIX_FMT_YUV411P",   ///< planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples)
    "AV_PIX_FMT_GRAY8",     ///<        Y        ,  8bpp
    "AV_PIX_FMT_MONOWHITE", ///<        Y        ,  1bpp, 0 is white, 1 is black, in each byte pixels are ordered from the msb to the lsb
    "AV_PIX_FMT_MONOBLACK", ///<        Y        ,  1bpp, 0 is black, 1 is white, in each byte pixels are ordered from the msb to the lsb
    "AV_PIX_FMT_PAL8",      ///< 8 bits with AV_PIX_FMT_RGB32 palette
    "AV_PIX_FMT_YUVJ420P",  ///< planar YUV 4:2:0, 12bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV420P and setting color_range
    "AV_PIX_FMT_YUVJ422P",  ///< planar YUV 4:2:2, 16bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV422P and setting color_range
    "AV_PIX_FMT_YUVJ444P",  ///< planar YUV 4:4:4, 24bpp, full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV444P and setting color_range
    "AV_PIX_FMT_UYVY422",   ///< packed YUV 4:2:2, 16bpp, Cb Y0 Cr Y1
    "AV_PIX_FMT_UYYVYY411", ///< packed YUV 4:1:1, 12bpp, Cb Y0 Y1 Cr Y2 Y3
    "AV_PIX_FMT_BGR8",      ///< packed RGB 3:3:2,  8bpp, (msb)2B 3G 3R(lsb)
    "AV_PIX_FMT_BGR4",      ///< packed RGB 1:2:1 bitstream,  4bpp, (msb)1B 2G 1R(lsb), a byte contains two pixels, the first pixel in the byte is the one composed by the 4 msb bits
    "AV_PIX_FMT_BGR4_BYTE", ///< packed RGB 1:2:1,  8bpp, (msb)1B 2G 1R(lsb)
    "AV_PIX_FMT_RGB8",      ///< packed RGB 3:3:2,  8bpp, (msb)2R 3G 3B(lsb)
    "AV_PIX_FMT_RGB4",      ///< packed RGB 1:2:1 bitstream,  4bpp, (msb)1R 2G 1B(lsb), a byte contains two pixels, the first pixel in the byte is the one composed by the 4 msb bits
    "AV_PIX_FMT_RGB4_BYTE", ///< packed RGB 1:2:1,  8bpp, (msb)1R 2G 1B(lsb)
    "AV_PIX_FMT_NV12",      ///< planar YUV 4:2:0, 12bpp, 1 plane for Y and 1 plane for the UV components, which are interleaved (first byte U and the following byte V)
    "AV_PIX_FMT_NV21",      ///< as above, but U and V bytes are swapped

    "AV_PIX_FMT_ARGB", ///< packed ARGB 8:8:8:8, 32bpp, ARGBARGB...
    "AV_PIX_FMT_RGBA", ///< packed RGBA 8:8:8:8, 32bpp, RGBARGBA...
    "AV_PIX_FMT_ABGR", ///< packed ABGR 8:8:8:8, 32bpp, ABGRABGR...
    "AV_PIX_FMT_BGRA", ///< packed BGRA 8:8:8:8, 32bpp, BGRABGRA...

    "AV_PIX_FMT_GRAY16BE", ///<        Y        , 16bpp, big-endian
    "AV_PIX_FMT_GRAY16LE", ///<        Y        , 16bpp, little-endian
    "AV_PIX_FMT_YUV440P",  ///< planar YUV 4:4:0 (1 Cr & Cb sample per 1x2 Y samples)
    "AV_PIX_FMT_YUVJ440P", ///< planar YUV 4:4:0 full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV440P and setting color_range
    "AV_PIX_FMT_YUVA420P", ///< planar YUV 4:2:0, 20bpp, (1 Cr & Cb sample per 2x2 Y & A samples)
    "AV_PIX_FMT_RGB48BE",  ///< packed RGB 16:16:16, 48bpp, 16R, 16G, 16B, the 2-byte value for each R/G/B component is stored as big-endian
    "AV_PIX_FMT_RGB48LE",  ///< packed RGB 16:16:16, 48bpp, 16R, 16G, 16B, the 2-byte value for each R/G/B component is stored as little-endian

    "AV_PIX_FMT_RGB565BE", ///< packed RGB 5:6:5, 16bpp, (msb)   5R 6G 5B(lsb), big-endian
    "AV_PIX_FMT_RGB565LE", ///< packed RGB 5:6:5, 16bpp, (msb)   5R 6G 5B(lsb), little-endian
    "AV_PIX_FMT_RGB555BE", ///< packed RGB 5:5:5, 16bpp, (msb)1X 5R 5G 5B(lsb), big-endian   , X=unused/undefined
    "AV_PIX_FMT_RGB555LE", ///< packed RGB 5:5:5, 16bpp, (msb)1X 5R 5G 5B(lsb), little-endian, X=unused/undefined

    "AV_PIX_FMT_BGR565BE", ///< packed BGR 5:6:5, 16bpp, (msb)   5B 6G 5R(lsb), big-endian
    "AV_PIX_FMT_BGR565LE", ///< packed BGR 5:6:5, 16bpp, (msb)   5B 6G 5R(lsb), little-endian
    "AV_PIX_FMT_BGR555BE", ///< packed BGR 5:5:5, 16bpp, (msb)1X 5B 5G 5R(lsb), big-endian   , X=unused/undefined
    "AV_PIX_FMT_BGR555LE", ///< packed BGR 5:5:5, 16bpp, (msb)1X 5B 5G 5R(lsb), little-endian, X=unused/undefined

    /**
     *  Hardware acceleration through VA-API, data[3] contains a
     *  VASurfaceID.
     */
    "AV_PIX_FMT_VAAPI",

    "AV_PIX_FMT_YUV420P16LE", ///< planar YUV 4:2:0, 24bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian
    "AV_PIX_FMT_YUV420P16BE", ///< planar YUV 4:2:0, 24bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
    "AV_PIX_FMT_YUV422P16LE", ///< planar YUV 4:2:2, 32bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
    "AV_PIX_FMT_YUV422P16BE", ///< planar YUV 4:2:2, 32bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian
    "AV_PIX_FMT_YUV444P16LE", ///< planar YUV 4:4:4, 48bpp, (1 Cr & Cb sample per 1x1 Y samples), little-endian
    "AV_PIX_FMT_YUV444P16BE", ///< planar YUV 4:4:4, 48bpp, (1 Cr & Cb sample per 1x1 Y samples), big-endian
    "AV_PIX_FMT_DXVA2_VLD",   ///< HW decoding through DXVA2, Picture.data[3] contains a LPDIRECT3DSURFACE9 pointer

    "AV_PIX_FMT_RGB444LE", ///< packed RGB 4:4:4, 16bpp, (msb)4X 4R 4G 4B(lsb), little-endian, X=unused/undefined
    "AV_PIX_FMT_RGB444BE", ///< packed RGB 4:4:4, 16bpp, (msb)4X 4R 4G 4B(lsb), big-endian",    X=unused/undefined
    "AV_PIX_FMT_BGR444LE", ///< packed BGR 4:4:4, 16bpp, (msb)4X 4B 4G 4R(lsb), little-endian, X=unused/undefined
    "AV_PIX_FMT_BGR444BE", ///< packed BGR 4:4:4, 16bpp, (msb)4X 4B 4G 4R(lsb), big-endian",    X=unused/undefined
    "AV_PIX_FMT_YA8",      ///< 8 bits gray, 8 bits alpha

    "AV_PIX_FMT_Y400A ", ///< alias for AV_PIX_FMT_YA8
    "AV_PIX_FMT_GRAY8A", ///< alias for AV_PIX_FMT_YA8

    "AV_PIX_FMT_BGR48BE", ///< packed RGB 16:16:16, 48bpp, 16B, 16G, 16R, the 2-byte value for each R/G/B component is stored as big-endian
    "AV_PIX_FMT_BGR48LE", ///< packed RGB 16:16:16, 48bpp, 16B, 16G, 16R, the 2-byte value for each R/G/B component is stored as little-endian

    /**
     * The following 12 formats have the disadvantage of needing 1 format for each bit depth.
     * Notice that each 9/10 bits sample is stored in 16 bits with extra padding.
     * If you want to support multiple bit depths, then using AV_PIX_FMT_YUV420P16* with the bpp stored separately is better.
     */
    "AV_PIX_FMT_YUV420P9BE",   ///< planar YUV 4:2:0, 13.5bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
    "AV_PIX_FMT_YUV420P9LE",   ///< planar YUV 4:2:0, 13.5bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian
    "AV_PIX_FMT_YUV420P10BE",  ///< planar YUV 4:2:0, 15bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
    "AV_PIX_FMT_YUV420P10LE",  ///< planar YUV 4:2:0, 15bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian
    "AV_PIX_FMT_YUV422P10BE",  ///< planar YUV 4:2:2, 20bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian
    "AV_PIX_FMT_YUV422P10LE",  ///< planar YUV 4:2:2, 20bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
    "AV_PIX_FMT_YUV444P9BE",   ///< planar YUV 4:4:4, 27bpp, (1 Cr & Cb sample per 1x1 Y samples), big-endian
    "AV_PIX_FMT_YUV444P9LE",   ///< planar YUV 4:4:4, 27bpp, (1 Cr & Cb sample per 1x1 Y samples), little-endian
    "AV_PIX_FMT_YUV444P10BE",  ///< planar YUV 4:4:4, 30bpp, (1 Cr & Cb sample per 1x1 Y samples), big-endian
    "AV_PIX_FMT_YUV444P10LE",  ///< planar YUV 4:4:4, 30bpp, (1 Cr & Cb sample per 1x1 Y samples), little-endian
    "AV_PIX_FMT_YUV422P9BE",   ///< planar YUV 4:2:2, 18bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian
    "AV_PIX_FMT_YUV422P9LE",   ///< planar YUV 4:2:2, 18bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
    "AV_PIX_FMT_GBRP",         ///< planar GBR 4:4:4 24bpp
    "AV_PIX_FMT_GBR24P",       // alias for #AV_PIX_FMT_GBRP
    "AV_PIX_FMT_GBRP9BE",      ///< planar GBR 4:4:4 27bpp, big-endian
    "AV_PIX_FMT_GBRP9LE",      ///< planar GBR 4:4:4 27bpp, little-endian
    "AV_PIX_FMT_GBRP10BE",     ///< planar GBR 4:4:4 30bpp, big-endian
    "AV_PIX_FMT_GBRP10LE",     ///< planar GBR 4:4:4 30bpp, little-endian
    "AV_PIX_FMT_GBRP16BE",     ///< planar GBR 4:4:4 48bpp, big-endian
    "AV_PIX_FMT_GBRP16LE",     ///< planar GBR 4:4:4 48bpp, little-endian
    "AV_PIX_FMT_YUVA422P",     ///< planar YUV 4:2:2 24bpp, (1 Cr & Cb sample per 2x1 Y & A samples)
    "AV_PIX_FMT_YUVA444P",     ///< planar YUV 4:4:4 32bpp, (1 Cr & Cb sample per 1x1 Y & A samples)
    "AV_PIX_FMT_YUVA420P9BE",  ///< planar YUV 4:2:0 22.5bpp, (1 Cr & Cb sample per 2x2 Y & A samples), big-endian
    "AV_PIX_FMT_YUVA420P9LE",  ///< planar YUV 4:2:0 22.5bpp, (1 Cr & Cb sample per 2x2 Y & A samples), little-endian
    "AV_PIX_FMT_YUVA422P9BE",  ///< planar YUV 4:2:2 27bpp, (1 Cr & Cb sample per 2x1 Y & A samples), big-endian
    "AV_PIX_FMT_YUVA422P9LE",  ///< planar YUV 4:2:2 27bpp, (1 Cr & Cb sample per 2x1 Y & A samples), little-endian
    "AV_PIX_FMT_YUVA444P9BE",  ///< planar YUV 4:4:4 36bpp, (1 Cr & Cb sample per 1x1 Y & A samples), big-endian
    "AV_PIX_FMT_YUVA444P9LE",  ///< planar YUV 4:4:4 36bpp, (1 Cr & Cb sample per 1x1 Y & A samples), little-endian
    "AV_PIX_FMT_YUVA420P10BE", ///< planar YUV 4:2:0 25bpp, (1 Cr & Cb sample per 2x2 Y & A samples, big-endian)
    "AV_PIX_FMT_YUVA420P10LE", ///< planar YUV 4:2:0 25bpp, (1 Cr & Cb sample per 2x2 Y & A samples, little-endian)
    "AV_PIX_FMT_YUVA422P10BE", ///< planar YUV 4:2:2 30bpp, (1 Cr & Cb sample per 2x1 Y & A samples, big-endian)
    "AV_PIX_FMT_YUVA422P10LE", ///< planar YUV 4:2:2 30bpp, (1 Cr & Cb sample per 2x1 Y & A samples, little-endian)
    "AV_PIX_FMT_YUVA444P10BE", ///< planar YUV 4:4:4 40bpp, (1 Cr & Cb sample per 1x1 Y & A samples, big-endian)
    "AV_PIX_FMT_YUVA444P10LE", ///< planar YUV 4:4:4 40bpp, (1 Cr & Cb sample per 1x1 Y & A samples, little-endian)
    "AV_PIX_FMT_YUVA420P16BE", ///< planar YUV 4:2:0 40bpp, (1 Cr & Cb sample per 2x2 Y & A samples, big-endian)
    "AV_PIX_FMT_YUVA420P16LE", ///< planar YUV 4:2:0 40bpp, (1 Cr & Cb sample per 2x2 Y & A samples, little-endian)
    "AV_PIX_FMT_YUVA422P16BE", ///< planar YUV 4:2:2 48bpp, (1 Cr & Cb sample per 2x1 Y & A samples, big-endian)
    "AV_PIX_FMT_YUVA422P16LE", ///< planar YUV 4:2:2 48bpp, (1 Cr & Cb sample per 2x1 Y & A samples, little-endian)
    "AV_PIX_FMT_YUVA444P16BE", ///< planar YUV 4:4:4 64bpp, (1 Cr & Cb sample per 1x1 Y & A samples, big-endian)
    "AV_PIX_FMT_YUVA444P16LE", ///< planar YUV 4:4:4 64bpp, (1 Cr & Cb sample per 1x1 Y & A samples, little-endian)

    "AV_PIX_FMT_VDPAU", ///< HW acceleration through VDPAU, Picture.data[3] contains a VdpVideoSurface

    "AV_PIX_FMT_XYZ12LE", ///< packed XYZ 4:4:4, 36 bpp, (msb) 12X, 12Y, 12Z (lsb), the 2-byte value for each X/Y/Z is stored as little-endian, the 4 lower bits are set to 0
    "AV_PIX_FMT_XYZ12BE", ///< packed XYZ 4:4:4, 36 bpp, (msb) 12X, 12Y, 12Z (lsb), the 2-byte value for each X/Y/Z is stored as big-endian, the 4 lower bits are set to 0
    "AV_PIX_FMT_NV16",    ///< interleaved chroma YUV 4:2:2, 16bpp, (1 Cr & Cb sample per 2x1 Y samples)
    "AV_PIX_FMT_NV20LE",  ///< interleaved chroma YUV 4:2:2, 20bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
    "AV_PIX_FMT_NV20BE",  ///< interleaved chroma YUV 4:2:2, 20bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian

    "AV_PIX_FMT_RGBA64BE", ///< packed RGBA 16:16:16:16, 64bpp, 16R, 16G, 16B, 16A, the 2-byte value for each R/G/B/A component is stored as big-endian
    "AV_PIX_FMT_RGBA64LE", ///< packed RGBA 16:16:16:16, 64bpp, 16R, 16G, 16B, 16A, the 2-byte value for each R/G/B/A component is stored as little-endian
    "AV_PIX_FMT_BGRA64BE", ///< packed RGBA 16:16:16:16, 64bpp, 16B, 16G, 16R, 16A, the 2-byte value for each R/G/B/A component is stored as big-endian
    "AV_PIX_FMT_BGRA64LE", ///< packed RGBA 16:16:16:16, 64bpp, 16B, 16G, 16R, 16A, the 2-byte value for each R/G/B/A component is stored as little-endian

    "AV_PIX_FMT_YVYU422", ///< packed YUV 4:2:2, 16bpp, Y0 Cr Y1 Cb

    "AV_PIX_FMT_YA16BE", ///< 16 bits gray, 16 bits alpha (big-endian)
    "AV_PIX_FMT_YA16LE", ///< 16 bits gray, 16 bits alpha (little-endian)

    "AV_PIX_FMT_GBRAP",     ///< planar GBRA 4:4:4:4 32bpp
    "AV_PIX_FMT_GBRAP16BE", ///< planar GBRA 4:4:4:4 64bpp, big-endian
    "AV_PIX_FMT_GBRAP16LE", ///< planar GBRA 4:4:4:4 64bpp, little-endian
    /**
     *  HW acceleration through QSV, data[3] contains a pointer to the
     *  mfxFrameSurface1 structure.
     */
    "AV_PIX_FMT_QSV",
    /**
     * HW acceleration though MMAL, data[3] contains a pointer to the
     * MMAL_BUFFER_HEADER_T structure.
     */
    "AV_PIX_FMT_MMAL",

    "AV_PIX_FMT_D3D11VA_VLD", ///< HW decoding through Direct3D11 via old API, Picture.data[3] contains a ID3D11VideoDecoderOutputView pointer

    /**
     * HW acceleration through CUDA. data[i] contain CUdeviceptr pointers
     * exactly as for system memory frames.
     */
    "AV_PIX_FMT_CUDA",

    "AV_PIX_FMT_0RGB", ///< packed RGB 8:8:8, 32bpp, XRGBXRGB...   X=unused/undefined
    "AV_PIX_FMT_RGB0", ///< packed RGB 8:8:8, 32bpp, RGBXRGBX...   X=unused/undefined
    "AV_PIX_FMT_0BGR", ///< packed BGR 8:8:8, 32bpp, XBGRXBGR...   X=unused/undefined
    "AV_PIX_FMT_BGR0", ///< packed BGR 8:8:8, 32bpp, BGRXBGRX...   X=unused/undefined

    "AV_PIX_FMT_YUV420P12BE", ///< planar YUV 4:2:0,18bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
    "AV_PIX_FMT_YUV420P12LE", ///< planar YUV 4:2:0,18bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian
    "AV_PIX_FMT_YUV420P14BE", ///< planar YUV 4:2:0,21bpp, (1 Cr & Cb sample per 2x2 Y samples), big-endian
    "AV_PIX_FMT_YUV420P14LE", ///< planar YUV 4:2:0,21bpp, (1 Cr & Cb sample per 2x2 Y samples), little-endian
    "AV_PIX_FMT_YUV422P12BE", ///< planar YUV 4:2:2,24bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian
    "AV_PIX_FMT_YUV422P12LE", ///< planar YUV 4:2:2,24bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
    "AV_PIX_FMT_YUV422P14BE", ///< planar YUV 4:2:2,28bpp, (1 Cr & Cb sample per 2x1 Y samples), big-endian
    "AV_PIX_FMT_YUV422P14LE", ///< planar YUV 4:2:2,28bpp, (1 Cr & Cb sample per 2x1 Y samples), little-endian
    "AV_PIX_FMT_YUV444P12BE", ///< planar YUV 4:4:4,36bpp, (1 Cr & Cb sample per 1x1 Y samples), big-endian
    "AV_PIX_FMT_YUV444P12LE", ///< planar YUV 4:4:4,36bpp, (1 Cr & Cb sample per 1x1 Y samples), little-endian
    "AV_PIX_FMT_YUV444P14BE", ///< planar YUV 4:4:4,42bpp, (1 Cr & Cb sample per 1x1 Y samples), big-endian
    "AV_PIX_FMT_YUV444P14LE", ///< planar YUV 4:4:4,42bpp, (1 Cr & Cb sample per 1x1 Y samples), little-endian
    "AV_PIX_FMT_GBRP12BE",    ///< planar GBR 4:4:4 36bpp, big-endian
    "AV_PIX_FMT_GBRP12LE",    ///< planar GBR 4:4:4 36bpp, little-endian
    "AV_PIX_FMT_GBRP14BE",    ///< planar GBR 4:4:4 42bpp, big-endian
    "AV_PIX_FMT_GBRP14LE",    ///< planar GBR 4:4:4 42bpp, little-endian
    "AV_PIX_FMT_YUVJ411P",    ///< planar YUV 4:1:1, 12bpp, (1 Cr & Cb sample per 4x1 Y samples) full scale (JPEG), deprecated in favor of AV_PIX_FMT_YUV411P and setting color_range

    "AV_PIX_FMT_BAYER_BGGR8",    ///< bayer, BGBG..(odd line), GRGR..(even line), 8-bit samples
    "AV_PIX_FMT_BAYER_RGGB8",    ///< bayer, RGRG..(odd line), GBGB..(even line), 8-bit samples
    "AV_PIX_FMT_BAYER_GBRG8",    ///< bayer, GBGB..(odd line), RGRG..(even line), 8-bit samples
    "AV_PIX_FMT_BAYER_GRBG8",    ///< bayer, GRGR..(odd line), BGBG..(even line), 8-bit samples
    "AV_PIX_FMT_BAYER_BGGR16LE", ///< bayer, BGBG..(odd line), GRGR..(even line), 16-bit samples, little-endian
    "AV_PIX_FMT_BAYER_BGGR16BE", ///< bayer, BGBG..(odd line), GRGR..(even line), 16-bit samples, big-endian
    "AV_PIX_FMT_BAYER_RGGB16LE", ///< bayer, RGRG..(odd line), GBGB..(even line), 16-bit samples, little-endian
    "AV_PIX_FMT_BAYER_RGGB16BE", ///< bayer, RGRG..(odd line), GBGB..(even line), 16-bit samples, big-endian
    "AV_PIX_FMT_BAYER_GBRG16LE", ///< bayer, GBGB..(odd line), RGRG..(even line), 16-bit samples, little-endian
    "AV_PIX_FMT_BAYER_GBRG16BE", ///< bayer, GBGB..(odd line), RGRG..(even line), 16-bit samples, big-endian
    "AV_PIX_FMT_BAYER_GRBG16LE", ///< bayer, GRGR..(odd line), BGBG..(even line), 16-bit samples, little-endian
    "AV_PIX_FMT_BAYER_GRBG16BE", ///< bayer, GRGR..(odd line), BGBG..(even line), 16-bit samples, big-endian

    "AV_PIX_FMT_XVMC", ///< XVideo Motion Acceleration via common packet passing

    "AV_PIX_FMT_YUV440P10LE", ///< planar YUV 4:4:0,20bpp, (1 Cr & Cb sample per 1x2 Y samples), little-endian
    "AV_PIX_FMT_YUV440P10BE", ///< planar YUV 4:4:0,20bpp, (1 Cr & Cb sample per 1x2 Y samples), big-endian
    "AV_PIX_FMT_YUV440P12LE", ///< planar YUV 4:4:0,24bpp, (1 Cr & Cb sample per 1x2 Y samples), little-endian
    "AV_PIX_FMT_YUV440P12BE", ///< planar YUV 4:4:0,24bpp, (1 Cr & Cb sample per 1x2 Y samples), big-endian
    "AV_PIX_FMT_AYUV64LE",    ///< packed AYUV 4:4:4,64bpp (1 Cr & Cb sample per 1x1 Y & A samples), little-endian
    "AV_PIX_FMT_AYUV64BE",    ///< packed AYUV 4:4:4,64bpp (1 Cr & Cb sample per 1x1 Y & A samples), big-endian

    "AV_PIX_FMT_VIDEOTOOLBOX", ///< hardware decoding through Videotoolbox

    "AV_PIX_FMT_P010LE", ///< like NV12, with 10bpp per component, data in the high bits, zeros in the low bits, little-endian
    "AV_PIX_FMT_P010BE", ///< like NV12, with 10bpp per component, data in the high bits, zeros in the low bits, big-endian

    "AV_PIX_FMT_GBRAP12BE", ///< planar GBR 4:4:4:4 48bpp, big-endian
    "AV_PIX_FMT_GBRAP12LE", ///< planar GBR 4:4:4:4 48bpp, little-endian

    "AV_PIX_FMT_GBRAP10BE", ///< planar GBR 4:4:4:4 40bpp, big-endian
    "AV_PIX_FMT_GBRAP10LE", ///< planar GBR 4:4:4:4 40bpp, little-endian

    "AV_PIX_FMT_MEDIACODEC", ///< hardware decoding through MediaCodec

    "AV_PIX_FMT_GRAY12BE", ///<        Y        , 12bpp, big-endian
    "AV_PIX_FMT_GRAY12LE", ///<        Y        , 12bpp, little-endian
    "AV_PIX_FMT_GRAY10BE", ///<        Y        , 10bpp, big-endian
    "AV_PIX_FMT_GRAY10LE", ///<        Y        , 10bpp, little-endian

    "AV_PIX_FMT_P016LE", ///< like NV12, with 16bpp per component, little-endian
    "AV_PIX_FMT_P016BE", ///< like NV12, with 16bpp per component, big-endian

    /**
     * Hardware surfaces for Direct3D11.
     *
     * This is preferred over the legacy AV_PIX_FMT_D3D11VA_VLD. The new D3D11
     * hwaccel API and filtering support AV_PIX_FMT_D3D11 only.
     *
     * data[0] contains a ID3D11Texture2D pointer, and data[1] contains the
     * texture array index of the frame as intptr_t if the ID3D11Texture2D is
     * an array texture (or always 0 if it's a normal texture).
     */
    "AV_PIX_FMT_D3D11",

    "AV_PIX_FMT_GRAY9BE", ///<        Y        , 9bpp, big-endian
    "AV_PIX_FMT_GRAY9LE", ///<        Y        , 9bpp, little-endian

    "AV_PIX_FMT_GBRPF32BE",  ///< IEEE-754 single precision planar GBR 4:4:4",     96bpp, big-endian
    "AV_PIX_FMT_GBRPF32LE",  ///< IEEE-754 single precision planar GBR 4:4:4",     96bpp, little-endian
    "AV_PIX_FMT_GBRAPF32BE", ///< IEEE-754 single precision planar GBRA 4:4:4:4, 128bpp, big-endian
    "AV_PIX_FMT_GBRAPF32LE", ///< IEEE-754 single precision planar GBRA 4:4:4:4, 128bpp, little-endian

    /**
     * DRM-managed buffers exposed through PRIME buffer sharing.
     *
     * data[0] points to an AVDRMFrameDescriptor.
     */
    "AV_PIX_FMT_DRM_PRIME",
    /**
     * Hardware surfaces for OpenCL.
     *
     * data[i] contain 2D image objects (typed in C as cl_mem, used
     * in OpenCL as image2d_t) for each plane of the surface.
     */
    "AV_PIX_FMT_OPENCL",

    "AV_PIX_FMT_GRAY14BE", ///<        Y        , 14bpp, big-endian
    "AV_PIX_FMT_GRAY14LE", ///<        Y        , 14bpp, little-endian

    "AV_PIX_FMT_GRAYF32BE", ///< IEEE-754 single precision Y, 32bpp, big-endian
    "AV_PIX_FMT_GRAYF32LE", ///< IEEE-754 single precision Y, 32bpp, little-endian

    "AV_PIX_FMT_YUVA422P12BE", ///< planar YUV 4:2:2,24bpp, (1 Cr & Cb sample per 2x1 Y samples), 12b alpha, big-endian
    "AV_PIX_FMT_YUVA422P12LE", ///< planar YUV 4:2:2,24bpp, (1 Cr & Cb sample per 2x1 Y samples), 12b alpha, little-endian
    "AV_PIX_FMT_YUVA444P12BE", ///< planar YUV 4:4:4,36bpp, (1 Cr & Cb sample per 1x1 Y samples), 12b alpha, big-endian
    "AV_PIX_FMT_YUVA444P12LE", ///< planar YUV 4:4:4,36bpp, (1 Cr & Cb sample per 1x1 Y samples), 12b alpha, little-endian

    "AV_PIX_FMT_NV24", ///< planar YUV 4:4:4, 24bpp, 1 plane for Y and 1 plane for the UV components, which are interleaved (first byte U and the following byte V)
    "AV_PIX_FMT_NV42", ///< as above, but U and V bytes are swapped

    /**
     * Vulkan hardware images.
     *
     * data[0] points to an AVVkFrame
     */
    "AV_PIX_FMT_VULKAN",

    "AV_PIX_FMT_Y210BE", ///< packed YUV 4:2:2 like YUYV422, 20bpp, data in the high bits, big-endian
    "AV_PIX_FMT_Y210LE", ///< packed YUV 4:2:2 like YUYV422, 20bpp, data in the high bits, little-endian

    "AV_PIX_FMT_X2RGB10LE", ///< packed RGB 10:10:10, 30bpp, (msb)2X 10R 10G 10B(lsb), little-endian, X=unused/undefined
    "AV_PIX_FMT_X2RGB10BE", ///< packed RGB 10:10:10, 30bpp, (msb)2X 10R 10G 10B(lsb), big-endian, X=unused/undefined
    "AV_PIX_FMT_X2BGR10LE", ///< packed BGR 10:10:10, 30bpp, (msb)2X 10B 10G 10R(lsb), little-endian, X=unused/undefined
    "AV_PIX_FMT_X2BGR10BE", ///< packed BGR 10:10:10, 30bpp, (msb)2X 10B 10G 10R(lsb), big-endian, X=unused/undefined

    "AV_PIX_FMT_P210BE", ///< interleaved chroma YUV 4:2:2, 20bpp, data in the high bits, big-endian
    "AV_PIX_FMT_P210LE", ///< interleaved chroma YUV 4:2:2, 20bpp, data in the high bits, little-endian

    "AV_PIX_FMT_P410BE", ///< interleaved chroma YUV 4:4:4, 30bpp, data in the high bits, big-endian
    "AV_PIX_FMT_P410LE", ///< interleaved chroma YUV 4:4:4, 30bpp, data in the high bits, little-endian

    "AV_PIX_FMT_P216BE", ///< interleaved chroma YUV 4:2:2, 32bpp, big-endian
    "AV_PIX_FMT_P216LE", ///< interleaved chroma YUV 4:2:2, 32bpp, liddle-endian

    "AV_PIX_FMT_P416BE", ///< interleaved chroma YUV 4:4:4, 48bpp, big-endian
    "AV_PIX_FMT_P416LE", ///< interleaved chroma YUV 4:4:4, 48bpp, little-endian

    "AV_PIX_FMT_NB" ///< number of pixel formats, DO NOT USE THIS if you want to link with shared libav* because the number of formats might differ between versions
};

short format_convert(int format)
{
    short sdl_format = false;
    switch (format)
    {
    case AV_SAMPLE_FMT_U8:
    case AV_SAMPLE_FMT_U8P:
        sdl_format = AUDIO_U8;
        break;
    case AV_SAMPLE_FMT_S16:
    case AV_SAMPLE_FMT_S16P:
        sdl_format = AUDIO_S16SYS;
        break;
    case AV_SAMPLE_FMT_S32:
    case AV_SAMPLE_FMT_S32P:
        sdl_format = AUDIO_S32SYS;
        break;
    case AV_SAMPLE_FMT_FLT:
    case AV_SAMPLE_FMT_FLTP:
    case AV_SAMPLE_FMT_DBL:
    case AV_SAMPLE_FMT_DBLP:
        sdl_format = AUDIO_F32SYS;
        break;
    default:
        printf("err fmt!=%d\n", format);
        return false;
    }
    return sdl_format;
}

char *av_pcm_clone(AVFrame *frame)
{
    assert(NULL != frame);

    int32_t bytes_per_sample = av_get_bytes_per_sample((enum AVSampleFormat)frame->format);
    char *p_cur_ptr = NULL, *pcm_data = NULL;
    if (bytes_per_sample <= 0)
        return NULL;

    int32_t frame_size = frame->channels * frame->nb_samples * bytes_per_sample;

    // 1.For packet sample foramt we just store pcm data in byte order.
    if (!av_sample_fmt_is_planar((enum AVSampleFormat)frame->format))
    { // linesize[0] maybe 0 or has padding bits,so calculate the real size by ourself.

        p_cur_ptr = pcm_data = (char *)malloc(frame_size);
        memcpy(p_cur_ptr, frame->data[0], frame_size);
    }
    else
    { // 2.For plane sample foramt, we must store pcm datas interleaved. [LRLRLR...LR].
        p_cur_ptr = pcm_data = (char *)malloc(frame_size);
        for (int i = 0; i < frame->nb_samples; ++i) // nb_samples 每个样本的采样数，左右声道各采样
        {
            for (int j = 0; j < frame->channels; j++)
            {
                memcpy(p_cur_ptr, frame->data[j] + i * bytes_per_sample, bytes_per_sample);
                p_cur_ptr += bytes_per_sample;
            }
        }
    }
    return pcm_data;
}
void MediaPlayer::free_avformat_ctx()
{
    avformat_free_context(m_avformat_ctx);
    m_avformat_ctx = nullptr;
}

bool MediaPlayer::open_file(const char *filename)
{
    // printf("avformat_open_input file:%s\n", filename);
    if (m_avformat_ctx)
    {
        free_avformat_ctx();
    }
    // 初始化媒体文件或媒体流的构成和基本信息
    m_avformat_ctx = avformat_alloc_context();
    // 打开媒体文件
    if (avformat_open_input(&m_avformat_ctx, filename, NULL, NULL) < 0)
    {
        free_avformat_ctx();
        return false;
    }

    av_dump_format(m_avformat_ctx, 0, filename, 0);
    // 查找流信息
    if (avformat_find_stream_info(m_avformat_ctx, NULL) < 0)
    {
        printf("Could not find stream information\n");
        free_avformat_ctx();
        return false;
    }
    // 查找视频解码器并打开解码器
    m_video_avdecoder = new MediaAVDecoder;
    if (!m_video_avdecoder->init(m_avformat_ctx, AVMEDIA_TYPE_VIDEO))
    {
        free_avformat_ctx();
        printf("open video decoder error\n");
        return false;
    }
    else
    {
        m_use_opt->pix_fmt = AV_PIX_FMT_YUV420P;
        m_use_opt->aspect_type = keep_aspect_ratio;
        m_sws_ctx = sws_getContext(
            m_video_avdecoder->m_avcodec_context->width,
            m_video_avdecoder->m_avcodec_context->height,
            m_video_avdecoder->m_avcodec_context->pix_fmt,
            m_video_avdecoder->m_avcodec_context->width,
            m_video_avdecoder->m_avcodec_context->height,
            AV_PIX_FMT_YUV420P,
            SWS_BICUBIC,
            NULL,
            NULL, // dst filter
            NULL  // param
        );
    }

    // 查找音频解码器并打开解码器
    m_audio_avdecoder = new MediaAVDecoder;
    if (!m_audio_avdecoder->init(m_avformat_ctx, AVMEDIA_TYPE_AUDIO))
    {
        free_avformat_ctx();
        printf("open audio decoder error\n");
        return false;
    }
    else
    {
        SDL_AudioSpec spec;
        spec.freq = m_audio_avdecoder->m_avcodec_context->sample_rate;
        spec.channels = (Uint8)m_audio_avdecoder->m_avcodec_context->ch_layout.nb_channels;
        spec.format = format_convert(m_audio_avdecoder->m_avcodec_context->sample_fmt);
        spec.silence = (Uint8)0;
        spec.samples = m_audio_avdecoder->m_avcodec_context->frame_size;
        spec.callback = NULL; // 使用push方式

        // 开启SDL音频渲染
        m_current_audio_deviceId = m_render_receive_obj->openAudioDevice(&spec);
        m_current_aduio_render_time = (double)av_gettime() / 1000000.0;
    }
    return true;
}
void MediaPlayer::close_file()
{
    avformat_close_input(&m_avformat_ctx);
}
bool MediaPlayer::start_play(const char *_file_name)
{
    stop_play();
    file_name = const_cast<char *>(_file_name);  
    if (!open_file(file_name))
    {
        return false;
    }
    m_sws_spec->filename = "媒体路径:" + std::string(_file_name);
    m_stop = false;
    // 启动线程
    m_demux_thread = std::thread(&MediaPlayer::demux_thread, this);
    m_audio_decode_thread = std::thread(&MediaPlayer::audio_decode_thread, this);
    m_video_decode_thread = std::thread(&MediaPlayer::video_decode_thread, this);
    m_video_render_thread = std::thread(&MediaPlayer::render_audio_thread, this);
    m_audio_render_thread = std::thread(&MediaPlayer::render_video_thread, this);
    m_theoretical_render_video_time = m_theoretical_render_audio_time = (double)av_gettime() / 1000000.0; // 初始化时钟（获取当前系统时间，然后根据pts叠加到该时间上的方式进行同步）
    m_status = v_state::playing;
    return true;
}
bool MediaPlayer::stop_play()
{
    m_stop = true;
    m_pause_condition_variable.notify_all();
    if (m_demux_thread.joinable())
    {
        m_demux_thread.join();
    }
    if (m_video_decode_thread.joinable())
    {
        m_pkt_vidoe_condition_variable.notify_all();
        m_video_decode_thread.join();
    }

    if (m_video_render_thread.joinable())
    {
        m_frame_video_condition_varible.notify_all();
        m_video_render_thread.join();
    }

    if (m_audio_decode_thread.joinable())
    {
        m_pkt_audio_condition_variable.notify_all();
        m_audio_decode_thread.join();
    }

    if (m_audio_render_thread.joinable())
    {
        m_audio_render_thread.join();
    }
    m_audio_frame_queue.clear();
    m_video_frame_queue.clear();
    m_video_frame_queue.clear();
    m_audio_packet_queue.clear();
    m_status = v_state::stop;
    close_file();
    return true;
}
void MediaPlayer::pause_resume_play()
{
    if (v_state::pause == m_status)
    {
        m_status = v_state::playing;
        m_pause_condition_variable.notify_all();
        SDL_PauseAudioDevice(m_current_audio_deviceId, 0);
    }

    else
    {
        m_status = v_state::pause;
        SDL_PauseAudioDevice(m_current_audio_deviceId, 1);
    }
}
void MediaPlayer::pause_or_resume()
{
    std::unique_lock<std::mutex> locker(m_pause_mutex);
    m_pause_condition_variable.wait(locker, [this]()
                                    {
		if (v_state::pause == m_status)
		{
			printf("demux_thread m_pause_condition_variable PlayStatus::Pause\n");
			return false;//阻塞线程
		}
		else
		{
			return true;//通过notify唤醒wait时，需要再次执行lamada，当lamada返回true才可以继续执行，否则，继续阻塞
		} });
}
void MediaPlayer::push_pkt_queue(MediaAVPacket *pkt)
{
    auto codecId = m_avformat_ctx->streams[pkt->m_pakcet->stream_index]->codecpar->codec_type;
    if (codecId == AVMEDIA_TYPE_VIDEO)
    {

        /* time_t t = time(0);
        char ch[64];
        strftime(ch, sizeof(ch), "%Y-%m-%d %H-%M-%S", localtime(&t)); //年-月-日 时-分-秒
        printf("Time:%s [demux_thread]-----m_video_packet_queue.push(pkt);\r",ch); */

        std::lock_guard<std::mutex> locker(m_pkt_video_queue_mutex);
        m_video_packet_queue.push_back(pkt);
        m_pkt_vidoe_condition_variable.notify_one();
    }
    else if (codecId == AVMEDIA_TYPE_AUDIO)
    {
        /* time_t t = time(0);
        char ch[64];
        strftime(ch, sizeof(ch), "%Y-%m-%d %H-%M-%S", localtime(&t)); //年-月-日 时-分-秒
        printf("[demux_thread]-----m_audio_packet_queue.push(pkt);\r",ch); */
        std::lock_guard<std::mutex> locker(m_pkt_audio_queue_mutex);
        m_audio_packet_queue.push_back(pkt);
        m_pkt_audio_condition_variable.notify_one();
    }
}
void MediaPlayer::demux_thread()
{
    while (!m_stop)
    {
        pause_or_resume();

        // 这段需要确保音视频同步后，才能正常工作。否则由于packet不够导致无法正常解码播放
        // printf("video packet queue size: %d, video frame queue size:%d, audio packet queue size:%d,audio frame queue size :%d\n",
        //       m_video_packet_queue.size(), m_video_frame_queue.size(), m_audio_packet_queue.size(), m_audio_frame_queue.size());
        {
            std::lock_guard<std::mutex> locker(m_pkt_video_queue_mutex);
            if (m_video_frame_queue.size() > PACKETSIZE)
            {
                continue;
            }
        }

        {
            std::lock_guard<std::mutex> locker(m_pkt_audio_queue_mutex);
            if (m_audio_frame_queue.size() > PACKETSIZE)
            {
                continue;
            }
        }
        MediaAVPacket *packet = new MediaAVPacket;
        int result = av_read_frame(m_avformat_ctx, packet->m_pakcet);
        if (result < 0)
        {
            if (result == AVERROR_EOF)
            {
                printf("file read end\n");
                break;
            }
        }
        push_pkt_queue(packet);
        std::this_thread::sleep_for(std::chrono::milliseconds(1));
    }
    m_demux_finish = true;
    printf("[finished]:void MediaPlayer::demux_thread()\n");
}

void MediaPlayer::video_decode(MediaAVPacket *video_pkt)
{
    int complete = 0;
    do
    {
        int ret = m_video_avdecoder->send_avpacket(video_pkt);
        if (ret < 0)
        {
            break; // 跳出内循环
        }
        else
        {
            MediaAVFrame *media_frame = new MediaAVFrame;
            complete = m_video_avdecoder->receive_avframe(media_frame);
            if (complete != AVERROR(EAGAIN) && !complete)
            {
                std::lock_guard<std::mutex> locker(m_video_frame_queue_mutex);
                m_video_frame_queue.push_back(media_frame);
                break;
            }
        }
    } while (true);
}
void MediaPlayer::video_decode_thread()
{
    while (!m_stop)
    {
        // deal pause operate
        pause_or_resume();
        MediaAVPacket *video_pkt = new MediaAVPacket;
        std::unique_lock<std::mutex> video_pkt_que_loker(m_pkt_video_queue_mutex);
        m_pkt_vidoe_condition_variable.wait(video_pkt_que_loker);
        if (!m_video_packet_queue.empty())
        {
            video_pkt = m_video_packet_queue.front();
            m_video_packet_queue.pop_front();
            video_decode(video_pkt);
            delete video_pkt;
            video_pkt = nullptr;
        }
        else
        {
            if (m_demux_finish)
            {
                m_video_decode_finish = true;
            }
        }
        // 视频解码完成
        if (m_video_decode_finish)
        {
            printf("video_decode_thread thread finished..................................\n");
            // flush 视频解码器缓存
            video_pkt->m_pakcet->data = NULL;
            video_pkt->m_pakcet->size = 0;
            video_decode(video_pkt);
            delete video_pkt;
            video_pkt = nullptr;
            break;
        }
    }
    printf("[finished]:void MediaPlayer::video_decode_thread()\n");
}
void MediaPlayer::audio_decode(MediaAVPacket *audio_pkt)
{
    int complete = 0;
    do
    {
        int ret = m_audio_avdecoder->send_avpacket(audio_pkt);
        if (ret < 0)
        {
            break;
        }
        else
        {
            MediaAVFrame *media_frame = new MediaAVFrame;
            complete = m_audio_avdecoder->receive_avframe(media_frame);
            if (complete != AVERROR(EAGAIN) && !complete)
            {
                std::unique_lock<std::mutex> locker(m_audio_frame_queue_mutex);
                m_audio_frame_queue.push_back(media_frame);
                break;
            }
        }
    } while (true);
}
void MediaPlayer::audio_decode_thread()
{
    while (!m_stop)
    {
        pause_or_resume();
        MediaAVPacket *audio_pkt = new MediaAVPacket;
        std::unique_lock<std::mutex> audio_que_loker(m_pkt_audio_queue_mutex);
        m_pkt_audio_condition_variable.wait(audio_que_loker);
        if (!m_audio_packet_queue.empty())
        {
            audio_pkt = m_audio_packet_queue.front();
            m_audio_packet_queue.pop_front();
            audio_decode(audio_pkt);
            delete audio_pkt;
            audio_pkt = nullptr;
        }
        else
        {
            if (m_demux_finish)
            {
                m_audio_decode_finish = true;
            }
        }
        // 解码完成，退出解码线程
        if (m_audio_decode_finish)
        {
            // 刷新解码器缓存
            audio_pkt->m_pakcet->data = NULL;
            audio_pkt->m_pakcet->size = 0;
            audio_decode(audio_pkt);
            delete audio_pkt;
            audio_pkt = nullptr;
            printf("audio_decodec thread finished..................\n");
            break;
        }
    }
    printf("[finished]:void MediaPlayer::audio_decode_thread()\n");
}
void MediaPlayer::render_video_thread()
{
    m_frameSize = av_image_get_buffer_size(m_video_avdecoder->m_avcodec_context->pix_fmt, m_video_avdecoder->m_avcodec_context->width, m_video_avdecoder->m_avcodec_context->height, 1);
    buffer = (uint8_t *)av_malloc(m_frameSize); // 指向YUV420的数据部分
    static double m_previous_pts_diff = 40e-3;
    static double m_previous_pts = 0;
    static bool m_first_frame = true;
    double delay_until_next_wake;
    while (!m_stop)
    {
        bool late_first_frame = false;
        pause_or_resume();
        if (m_video_decode_finish)
        {
            break;
        }

        std::unique_lock<std::mutex> locker(m_video_frame_queue_mutex);
        if (m_video_frame_queue.empty())
        {
            locker.unlock();
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
            continue;
        }
        if (m_video_af_frame)
        {
            delete m_video_af_frame;
            m_video_af_frame = nullptr;
        }
        m_video_af_frame = new MediaAVFrame;
        m_video_af_frame = m_video_frame_queue.front();
        m_video_frame_queue.pop_front();
        locker.unlock();

        auto videoPts = m_video_af_frame->m_frame->best_effort_timestamp * av_q2d(m_video_avdecoder->m_avstream->time_base);
        double pts_diff = videoPts - m_previous_pts;
        if (m_first_frame)
        {
            late_first_frame = pts_diff >= 1.0;
            m_first_frame = false;
        }
        if (pts_diff <= 0 || late_first_frame)
        {
            pts_diff = m_previous_pts_diff;
        }
        m_previous_pts_diff = pts_diff;
        m_previous_pts = videoPts;

        auto delay = av_gettime() - m_current_aduio_render_time; // 渲染到现在为止的延迟
        int currentPts = m_audio_current_pts + delay;            // 当前音频的播放pts
        auto diff = videoPts - currentPts;                       // 【计算视频帧pts与当前音频帧pts的差值，使用这个值来判断是否要展示当前的视频帧】
        double sync_threshold;

        // 更新阈值
        sync_threshold = (pts_diff > AV_SYNC_THRESHOLD)
                             ? pts_diff
                             : AV_SYNC_THRESHOLD;

        // 修正pts_diff
        if (abs(diff) < AV_NOSYNC_THRESHOLD)
        {
            if (diff < -sync_threshold) // 如果diff小于0（表明视频帧落后），且超出了阈值，则立即播放
            {
                pts_diff = 0;
            }
            else if (diff > sync_threshold) // diff大于阈值，则表明当前视频帧没有放完,则延迟两个pts_diff
            {
                pts_diff = 2 * pts_diff;
            }
        }
        m_theoretical_render_video_time += pts_diff;

        delay_until_next_wake = m_theoretical_render_video_time - (av_gettime() / 1000000.0L);
        if (delay_until_next_wake < 0.010L)
        {
            delay_until_next_wake = 0.010L;
        }

        printf("render video delay:%lf\r", delay_until_next_wake);
        auto wake_tp = std::chrono::high_resolution_clock::now() + std::chrono::duration<int, std::micro>((int)(delay_until_next_wake * 1000000 + 500));

        std::this_thread::sleep_until(wake_tp);
        if (m_video_ot_frame)
        {
            delete m_video_ot_frame;
            m_video_ot_frame = nullptr;
        }
        m_video_ot_frame = new MediaAVFrame;

        // 将yuv_frame->m_frame->data数组指向buffer，并按照codecContext和AVPixelFormat决定data指针数组各个成员的指向
        av_image_fill_arrays(m_video_ot_frame->m_frame->data,               // dst data[]
                             m_video_ot_frame->m_frame->linesize,           // dst linesize[]
                             buffer,                                        // src buffer
                             m_video_avdecoder->m_avcodec_context->pix_fmt, // pixel format
                             m_video_avdecoder->m_avcodec_context->width,   // width
                             m_video_avdecoder->m_avcodec_context->height,  // height
                             1                                              // align
        );

        m_video_ot_frame->m_frame->width = m_video_avdecoder->m_avcodec_context->width;
        m_video_ot_frame->m_frame->height = m_video_avdecoder->m_avcodec_context->height;
        // sws_scale将各种video_format转换为AV_PIX_FMT_YUV420P
        sws_scale(m_sws_ctx,                                    // sws context
                  m_video_af_frame->m_frame->data,              // src slice
                  m_video_af_frame->m_frame->linesize,          // src stride
                  0,                                            // src slice y
                  m_video_avdecoder->m_avcodec_context->height, // src slice height
                  m_video_ot_frame->m_frame->data,              // dst planes
                  m_video_ot_frame->m_frame->linesize);         // dst strides

        if (m_sws_spec->info_state)
        {
            out_media_info();
        }

        QMetaObject::invokeMethod(
            QApplication::instance(), [=]()
            { m_render_receive_obj->update_image(m_video_ot_frame, m_sws_spec, m_use_opt); },
            Qt::QueuedConnection);
    }
    av_free(buffer);
    printf("[finished]:void MediaPlayer::render_video_thread()\n");
}
void MediaPlayer::render_audio_thread()
{
    while (!m_stop)
    {
        pause_or_resume();
        bool late_first_frame = false;
        std::unique_lock<std::mutex> locker(m_audio_frame_queue_mutex);
        static double m_previous_pts_diff = 40e-3;
        static bool m_first_frame = true;
        if (!m_audio_frame_queue.empty())
        {
            if (m_audio_af_frame)
            {
                delete m_audio_af_frame;
                m_audio_af_frame = nullptr;
            }
            m_audio_af_frame = new MediaAVFrame;
            m_audio_af_frame = m_audio_frame_queue.front();
            m_audio_frame_queue.pop_front();
            locker.unlock();

            m_audio_current_pts = m_audio_af_frame->m_frame->best_effort_timestamp * av_q2d(m_audio_avdecoder->m_avstream->time_base);

            // the amount of time until we need to display this frame
            double diff = m_audio_current_pts - m_previous_audio_pts;
            if (m_first_frame)
            {
                late_first_frame = diff >= 1.0;
                m_first_frame = false;
            }

            if (diff <= 0 || late_first_frame)
            { // if diff is invalid, use previous
                diff = m_previous_pts_diff;
            }

            // save for next time
            m_previous_audio_pts = m_audio_current_pts;
            m_previous_pts_diff = diff;

            m_theoretical_render_audio_time += diff; // 理论上应该渲染的time_point
            double timeDiff = m_theoretical_render_audio_time - (av_gettime() / 1000000.0L);

            if (timeDiff < 0.010L)
            {
                timeDiff = 0.01L;
            }

            if (timeDiff > diff)
            {
                timeDiff = diff;
            }

            // printf("audio frame will delay:%lf \n", timeDiff);
            auto wake_tp = std::chrono::high_resolution_clock::now() + std::chrono::duration<int, std::micro>((int)(timeDiff * 1000000));

            std::this_thread::sleep_until(wake_tp);

            int32_t bytes_per_sample = av_get_bytes_per_sample((enum AVSampleFormat)m_audio_af_frame->m_frame->format);
            int32_t size = m_audio_af_frame->m_frame->nb_samples * bytes_per_sample * m_audio_af_frame->m_frame->channels;
            char *data = av_pcm_clone(m_audio_af_frame->m_frame);
            m_current_aduio_render_time = m_audio_af_frame->m_frame->pts * av_q2d(m_audio_avdecoder->m_avstream->time_base);;
            // 音频时钟无效判断
            if (m_current_aduio_render_time != AV_NOPTS_VALUE)
            {                                                   
                emit signals_play_time_changed(this);
            }
            SDL_QueueAudio(m_current_audio_deviceId, data, size);
            free((void *)data);
        }
        else
        {
            locker.unlock();
            // printf("audio frame_queue is empty\n");
            std::this_thread::sleep_for(std::chrono::milliseconds(1));
        }
        if (m_audio_decode_finish)
        {
            break;
        }
    }
    printf("[finished]:void MediaPlayer::render_audio_thread()\n");
}
void MediaPlayer::out_media_info()
{
    int audioindex = -1;
    int videoindex = -1;
    for (int i = 0; i < m_avformat_ctx->nb_streams; i++)
    {
        if (m_avformat_ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_VIDEO)
        {
            videoindex = i;
        }
        else if (m_avformat_ctx->streams[i]->codecpar->codec_type == AVMEDIA_TYPE_AUDIO)
        {
            audioindex = i;
        }
    }
    m_sws_spec->width = "帧宽度:" + std::to_string(m_video_avdecoder->m_avcodec_context->width);
    m_sws_spec->height = "帧高度:" + std::to_string(m_video_avdecoder->m_avcodec_context->height);
    m_sws_spec->channels = "音频声道数:" + (audioindex == -1 ? "无法获取音频信息" : std::to_string(m_avformat_ctx->streams[audioindex]->codecpar->channels));
    m_sws_spec->sample_rate = "音频采样率:" + (audioindex == -1 ? "无法获取音频信息" : std::to_string(m_avformat_ctx->streams[audioindex]->codecpar->sample_rate));
    m_sws_spec->bit_rate = "码率:" + std::to_string(m_avformat_ctx->bit_rate);
    m_sws_spec->current_fps_rate = "当前帧率:" + std::to_string(m_video_avdecoder->m_avcodec_context->framerate.num);
    m_sws_spec->duration = "总时长:" + std::to_string(round(m_avformat_ctx->duration * av_q2d(AV_TIME_BASE_Q))) + "秒";
    m_sws_spec->pix_fmt = "视频像素格式:" + (videoindex == -1 ? "无法获取视频信息" : std::string(CharAVPixelFormat[m_avformat_ctx->streams[videoindex]->codecpar->frame_size + 1]));
    m_sws_spec->pkg_fmt_name = "封装格式名称:" + std::string(m_avformat_ctx->iformat->name);
}
void MediaPlayer::get_video_size(int &width, int &height)
{
    if (m_video_avdecoder->m_avcodec_context)
    {
        width = m_video_avdecoder->m_avcodec_context->width;
        height = m_video_avdecoder->m_avcodec_context->height;
    }
}
void MediaPlayer::set_video_size(int _width, int _height)
{
    if (m_use_opt)
    {
        pause_or_resume();
        m_use_opt->width = _width;
        m_use_opt->height = _height;
    }
}
v_sws_spec *MediaPlayer::get_sws_specs()
{
    return m_sws_spec;
}
void MediaPlayer::set_sws_specs(v_sws_spec *_m_sws_spec)
{
    m_sws_spec = _m_sws_spec;
}
v_state MediaPlayer::get_state()
{
    return m_status;
}
AVFormatContext *MediaPlayer::get_fmt_ctx()
{
    return m_avformat_ctx;
}
char *MediaPlayer::get_file_name()
{
    return file_name;
}
void MediaPlayer::register_render_windows_callback(SDL2RenderWidget *receiver)
{
    m_render_receive_obj = receiver;
}
int MediaPlayer::get_duration()
{
    return m_avformat_ctx ? round(m_avformat_ctx->duration * av_q2d(AV_TIME_BASE_Q)) : 0; // 时间转化为秒
}
int MediaPlayer::get_time()
{
    return round(m_current_aduio_render_time);
}