#ifndef MEDIAPALYER_H
#define MEDIAPALYER_H

#include <stdio.h>
extern "C"
{
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavutil/avutil.h>
#include <libswscale/swscale.h>
#include <libavutil/time.h>
#include <libavutil/imgutils.h>
};
#include "MediaAVDecoder.h"
#include "MediaAVStream.h"
#include "MediaAVPacket.h"
#include "MediaFFex.h"
#include "SDL2/SDL.h"
#include "SDL2RenderWidget.h"
#include <thread>
#include <condition_variable>
#include <mutex>
#include <deque>
#include <iostream>
#include <future>

#include <QApplication>
#include <QThread>
#define PACKETSIZE 60
#define AV_SYNC_THRESHOLD 0.01
#define AV_NOSYNC_THRESHOLD 10.0


class MediaPlayer:public QObject
{
    Q_OBJECT
public:
    MediaPlayer();
    ~MediaPlayer();
    bool             start_play(const char *_filename);
    bool             stop_play();
    void             pause_resume_play();
    void             set_video_size(int _width,int _height);
    void             get_video_size(int &_width, int &_height);
    void             set_sws_specs(v_sws_spec*);
    v_sws_spec*      get_sws_specs();
    v_state          get_state();
    AVFormatContext* get_fmt_ctx();
    char*            get_file_name();
    int              get_duration();
    int              get_time();
public:
    void register_render_windows_callback(SDL2RenderWidget *receiver);
private:
    bool open_file(const char *filename);
    void close_file();
    void pause_or_resume();
    void free_avformat_ctx();
    void out_media_info();
    void demux_thread();
    void push_pkt_queue(MediaAVPacket* pkt);
    void video_decode(MediaAVPacket* video_pkt);
    void audio_decode(MediaAVPacket* audio_pkt);
    void audio_decode_thread();
    void video_decode_thread();
    void render_audio_thread();
    void render_video_thread();
    void render_default_image_thread();
    
private:
    char* file_name;
    AVFormatContext*  m_avformat_ctx;
    MediaAVDecoder*   m_video_avdecoder;
    MediaAVDecoder*   m_audio_avdecoder;
    SwsContext*       m_sws_ctx;
    v_sws_spec*       m_sws_spec;
    use_opt*          m_use_opt;
    MediaAVFrame*     m_video_af_frame;
    MediaAVFrame*     m_video_ot_frame;
    MediaAVFrame*     m_audio_af_frame;
    MediaAVFrame*     m_audio_ot_frame;
    SDL2RenderWidget *m_render_receive_obj;
    SDL_AudioDeviceID m_current_audio_deviceId;

    std::thread m_demux_thread;
    std::thread m_video_decode_thread;
    std::thread m_audio_decode_thread;
    std::thread m_video_render_thread;
    std::thread m_video_default_image_thread;
    std::thread m_audio_render_thread;

    std::mutex m_pause_mutex;
    std::mutex m_pkt_audio_queue_mutex;
    std::mutex m_pkt_video_queue_mutex;
    std::mutex m_audio_frame_queue_mutex;
    std::mutex m_video_frame_queue_mutex;

    std::condition_variable m_pause_condition_variable;
    std::condition_variable m_demuex_condition_variable;
    std::condition_variable m_pkt_audio_condition_variable;
    std::condition_variable m_pkt_vidoe_condition_variable;
    std::condition_variable m_frame_video_condition_varible;

    std::deque<MediaAVPacket*> m_video_packet_queue;
    std::deque<MediaAVFrame*> m_video_frame_queue;
    std::deque<MediaAVPacket*> m_audio_packet_queue;
    std::deque<MediaAVFrame*> m_audio_frame_queue;

    bool m_demux_finish = false;
    bool m_audio_decode_finish = false;
    bool m_video_decode_finish = false;
    
    int m_frameSize,n_frameSize;
    uint8_t *buffer;
    bool m_stop = false;
    v_state m_status = v_state::none;
    double m_theoretical_render_audio_time;
    double m_theoretical_render_video_time;
    double m_current_aduio_render_time;
    double m_previous_audio_pts = 0;
    double m_audio_current_pts = 0;
signals:
    void signals_play_time_changed(MediaPlayer*);
};
#endif