#include "ZFFmpeg.h"

ZFFmpeg::ZFFmpeg(QWidget *parent)
    : QWidget(parent),
      widget_player(new QWidget),
      layout_main(new QVBoxLayout),
      layout_player(new QVBoxLayout),
      layout_player_state(new QHBoxLayout),
      layout_player_tool(new QHBoxLayout),
      slider_time(new MediaSlider),
      label_current_time(new QLabel),
      label_n(new QLabel),
      label_duration_time(new QLabel),
      btn_play_reverse3s(new QPushButton),
      btn_play_last(new QPushButton),
      btn_play(new QPushButton),
      btn_play_next(new QPushButton),
      btn_play_speed3s(new QPushButton),
      btn_stop(new QPushButton),
      btn_open_file(new QPushButton),
      btn_open_dir(new QPushButton),
      mutipleSpeed(new QComboBox),
      btn_mutex_volumn(new QPushButton),
      slider_volumn(new MediaSlider)
{
    setMinimumSize(600, 400);
    btn_play_reverse3s->setIcon(QIcon(tr(":/res/btn/reverse3s.png")));
    btn_play_last->setIcon(QIcon(tr(":/res/btn/lastMedia.png")));
    btn_play->setIcon(QIcon(tr(":/res/btn/play.png")));
    btn_play_next->setIcon(QIcon(tr(":/res/btn/nextMedia.png")));
    btn_play_speed3s->setIcon(QIcon(tr(":/res/btn/speed3s.png")));
    btn_stop->setIcon(QIcon(tr(":/res/btn/stop.png")));
    btn_open_file->setIcon(QIcon(tr(":/res/btn/addFile.png")));
    btn_open_dir->setIcon(QIcon(tr(":/res/btn/addFolder.png")));
    btn_mutex_volumn->setIcon(QIcon(tr(":/res/btn/loud.png")));
    slider_volumn->setOrientation(Qt::Orientation::Horizontal);

    layout_player_tool->addWidget(btn_play_reverse3s);
    layout_player_tool->addWidget(btn_play_last);
    layout_player_tool->addWidget(btn_play);
    layout_player_tool->addWidget(btn_play_next);
    layout_player_tool->addWidget(btn_play_speed3s);
    layout_player_tool->addWidget(btn_stop);
    layout_player_tool->addWidget(btn_open_file);
    layout_player_tool->addWidget(btn_open_dir);
    layout_player_tool->addSpacerItem(sparcer_item);
    layout_player_tool->addWidget(btn_mutex_volumn);
    layout_player_tool->addWidget(slider_volumn);
    layout_player_tool->setSpacing(0);

    label_current_time->setText(getTimeText(0));
    label_n->setText(tr("/"));
    label_duration_time->setText(getTimeText(0));
    slider_time->setOrientation(Qt::Orientation::Horizontal);

    layout_player_state->addWidget(slider_time);
    layout_player_state->addWidget(label_current_time);
    layout_player_state->addWidget(label_n);
    layout_player_state->addWidget(label_duration_time);
    layout_player_state->setSpacing(0);

    widget_player->setContentsMargins(0, 0, 0, 0);
    widget_sdl2 = new SDL2RenderWidget(widget_player);
    m_player = new MediaPlayer;
    m_player->register_render_windows_callback(widget_sdl2);
    m_player->set_video_size(size().width(),size().height());
    layout_player->addWidget(widget_player);

    layout_main->addLayout(layout_player);
    layout_main->addLayout(layout_player_state);
    layout_main->addLayout(layout_player_tool);
    layout_main->setStretch(0, 8);
    layout_main->setStretch(1, 1);
    layout_main->setStretch(2, 1);
    layout_main->setSpacing(0);
    setLayout(layout_main);
    

    connect(btn_play, SIGNAL(clicked()), this, SLOT(play()));
    connect(btn_play, SIGNAL(clicked()), this, SLOT(pause()));
    connect(btn_stop, SIGNAL(clicked()), this, SLOT(stop()), Qt::QueuedConnection);
    connect(btn_open_file, SIGNAL(clicked()), this, SLOT(set_open_file()));
    connect(widget_sdl2, SIGNAL(signals_change_size(int,int)), this, SLOT(slots_change_size(int,int)));
    connect(m_player, SIGNAL(signals_play_time_changed(MediaPlayer*)),this, SLOT(slots_play_time_changed(MediaPlayer*)));
    connect(m_player, SIGNAL(signals_play_time_changed(MediaPlayer*)),this, SLOT(slots_on_time_slider_value_changed(MediaPlayer*)));
}

ZFFmpeg::~ZFFmpeg()
{
}
void ZFFmpeg::keyReleaseEvent(QKeyEvent *event)
{
    if (event->key() == Qt::Key_Tab)
    {
        auto _specs=m_player->get_sws_specs();
        if(_specs->info_state){
            _specs->info_state=false;

        }else{
            _specs->info_state=true;
        }
        m_player->set_sws_specs(_specs);
    }
}
void ZFFmpeg::resizeEvent(QResizeEvent *event)
{
    widget_sdl2->set_Size(size());
    m_player->set_video_size(size().width(),size().height());
}
void ZFFmpeg::slots_change_size(int _w,int _h)
{
    this->resize(_w,_h);
}
QString ZFFmpeg::getTimeText(int value) {
    //获取XX：XX：XX格式的时间文本
    QLatin1Char fill = QLatin1Char('0');
    return QString("%1:%2:%3")
           .arg(value/3600,2,10,fill)
           .arg((value/60)%60,2,10,fill)
           .arg(value%60,2,10,fill);
}
void ZFFmpeg::set_open_file()
{
    ms_Path = QFileDialog::getOpenFileName(this, tr("选择要播放的文件"), tr(""));
    play();
}
void ZFFmpeg::slots_play_time_changed(MediaPlayer* _player) {
    slider_time->setValue(_player->get_time());
}

void ZFFmpeg::slots_on_time_slider_value_changed(MediaPlayer* _player) {
    label_current_time->setText(getTimeText(_player->get_time()));
}
void ZFFmpeg::play()
{
    if (ms_Path != nullptr)
    {
        m_player->start_play(ms_Path.toUtf8().data());
        int duration = m_player->get_duration();
        slider_time->setRange(0, duration);
        label_duration_time->setText(getTimeText(duration));
    }
}

void ZFFmpeg::pause()
{
}

void ZFFmpeg::stop()
{
}